## 👀 About MMSearch

The capabilities of **Large Multi-modal Models (LMMs)** in **multimodal search** remain insufficiently explored and evaluated. To fill the blank of a framework for LMM to conduct multimodal AI search engine, we first design a delicate pipeline **MMSearch-Engine** to facilitate **any LMM** to function as a multimodal AI search engine

<p align="center">
    <img src="figs/fig1.png" width="90%"> <br>
</p>

To further evaluate the potential of LMMs in the multimodal search domain, we introduce **MMSearch**, an all-around multimodal search benchmark designed for assessing the multimodal search performance. The benchmark contains 300 manually collected instances spanning 14 subfields, which involves no overlap with the current LMMs' training data, ensuring the correct answer can only be obtained within searching.

<p align="center">
    <img src="figs/fig2.png" width="90%"> <br>
    An overview of <b>MMSearch</b>.
</p>

In addition, we propose a **step-wise evaluation strategy** to better understand the LMMs' searching capability. The models are evaluated by performing **three individual tasks (requery, rerank, and summarization)**, and **one challenging end-to-end task** with a complete searching process. The final score is weighted by the four tasks.

<p align="center">
    <img src="figs/fig3.png" width="90%"> <br>
    Outline of Evaluation Tasks, Inputs, and Outputs.
</p>

<details>
<summary>🔍 An example of LMM input, output, and ground truth for four evaluation tasks</summary>

<p align="center">
    <img src="figs/fig4.png" width="50%"> <br>
</p>
</details>



## Demo
We provide a command line demo of MMSearch-Engine for any new queries. 

### Setup Environment 

The environment is mainly for interacting with the search engine and crawling the website:

```bash
pip install requirements.txt
playwright install
```

#### Get your LMMs ready

**(a). ✨ Use models implemented in [VLMEvalKit](https://github.com/open-compass/VLMEvalKit)**

We now support directly using the models implemented in [VLMEvalKit](https://github.com/open-compass/VLMEvalKit). You need to first install VLMEvalKit with the following command, or follow the guidance in its repo:

```bash
git clone https://github.com/open-compass/VLMEvalKit.git
cd VLMEvalKit
pip install -e .
```

Then, you can directly use the models implemented in VLMEvalKit, the available name list of the model is [here](https://github.com/open-compass/VLMEvalKit/blob/main/vlmeval/config.py). 

To use the model, simply add the prefix `vlmevalkit_` in front of the model name in the list. For example, to use `llava_onevision_qwen2_7b_ov`, your input `model_type` should be `vlmevalkit_llava_onevision_qwen2_7b_ov`. We provide an example of the rerank task in `scripts/run_requery_vlmevalkit.sh`.

**Note that, several models in VLMEvalKit do not support text-only inference, so it may not support end2end task (some queries in round1 do not have image input).**

**(b). 💪 Use custom LMMs**

Here, **we support evaluation of any custom LMMs with only very little effort**. To evaluate your LMM, you only need to provide an `infer` function, which takes the image files and text instructions as input and outputs the model response.

We implement the code of LLaVA-OneVision in `models/llava_model.py`. **Adding a model is very simple with only two steps**:

1. Implement a class for the model. The model class must implement the `infer` function, which takes image files and text instructions as input. Please refer to `models/llava_model.py` for the illustration of input variable types.
2. Add the model type in ``models/load.py``. Then you can specify the `model_type` in your bash file and use your model!

### Prepare query
We provide query examples in `demo/query_cli.json`. For queries with image, you need to specify the path to the `query_image` and an URL of the `query_image` since Google Lens here only supports url input. An easy way to get an URL of an image is to upload it to any public GitHub repository. Then simply substitute `blob` with `raw` of the image URL:
```
{
    "query": "When is the US release date for this movie?",
    "query_image": "demo/demo.png",
    "query_image_url": TO_BE_FILLED
}
```
For queries without image, you only need to specify the query and set `query_image` as `null`:
```
{
    "query": "When is the US release date for Venom: The Last Dance?",
    "query_image": null
}
```
### Get the search result
To successfully search the image in Google Lens, make sure the search engine that [playwright](https://github.com/microsoft/playwright) opens is in English. Otherwise, it will throw an error.
To get the search result of your queries, simply run the following command. The parameters have the same meaning as the parameters in the end2end evaluation task.

```
bash demo/run_demo_cli.sh
```
